# Multi-Tab Editor

The simple Editor is based on the `Ace Editor` [https://ace.c9.io/](https://ace.c9.io/) and is included in the package via the `brace` package.

There is nothing important to note about the simple Editor, no additional code is needed.
Just use the `simpleCodeEditorWrapper`


For all Editors is is important that the `id` is unique!
If we use the `onChange` event of the editor then the performance is very poor on low spec pcs because every keystroke fires a new event.
Thus we use the `onBlur` event with an additional storage for all ace editor instances (`ace storage`).

## The Ace Storage

If we conditional render an editor then we need to restore the state when we render the editor again
This is needed when we display modals or when the user gets automatically logged out after X min.

Because of this we use a `ace storage` where we store the tabs with the related ace session objects.
The Helper is named `AceEditorsStorageHelper`.

Whenever we fill the an editor with tabs we need to register the tabs to the `AceEditorsStorageHelper` which will then generate ace
sessions for the tabs. The tabs are also stored but only some props (e.g. id, language, ...)(we call them fake tabs or ace tabs) so that we can later commit the changes from the sessions to the real editor tab.
All changes are done against the sessions and in the `onBlur` event we update the ace tab.

So whenever the tabs in the editor change (add, delete or the tab id change because we saved the tab to the server) we need to update the `AceEditorsStorageHelper` ace tabs too else we are out of sync and cannot longer find the real editor tab to commit the changes.


## Using the Multi-Tab Editor

The wrapper for the multi-tab editor is `multiTabCodeEditorWrapper`.

>We use wrappers in case we move to another editor at some point.

Every multi-tab editor needs it's own storage to handle file creation, renaming, ...
An example can be found in `doEditorReducer.ts` and `doEditorActions.ts`

When the editor id is needed by other files it is a good idea to move the id to the `constants` file

Adding & deleting files is internally handled by the multi-tab editor

```tsx
<MultiTabCodeEditorWrapper
	id={UniqueEditorId}
	height="100%"
	... implement props
/>
```


but there are some cases where you need to manually update the `AceEditorsStorageHelper` for the editor

In the `doEditor` we have the reset code feature with will replace the user solution with the code template.
Because this replaces the editor files we also need to update the `AceEditorsStorageHelper`
This is done in the action (because there we create the new solution files).

After we created the new solution files we call

```ts

dispatch(setSolutionFiles(newSolutionFiles))
...

AceEditorsStorageHelper.removeEditor(doEditorEditorPanelViewEditorId)
```

to reset the storage and then create the new tabs

```ts
const storedTabs= newSolutionFiles.map<AceStoredTabLike>(p => {
	return {
		id: p.id,
		content: p.content,
		language: pLang.internalName
	}
})
```

the storage only needs to know the id, the language (and the content?)
afterwards we set the new tabs with

```ts
AceEditorsStorageHelper.addEditorWithTabs(doEditorEditorPanelViewEditorId, storedTabs)
```

We assume that the `newSolutionFiles` are attached to the editor via `editorTabs`

sometimes the editor won't rerender (e.g. if you don't update any other property in the action)
so it can happen that the editor will keep the old session because the check in did update
 wrapperTab.session === this.aceEditorWrapper.editor.getSession() is true
because the session is old (at this time we updated the editor tabs because we updated the files) and the editor is still the same (with the same session)
setting AceEditorsStorageHelper.addEditorWithTabs won't trigger react to rerender the editor and call the did update again

so it's sometimes necessary to set some other prop that is connected to the editor to force a rerender e.g. with setting the main file or something

---

Another scenario is where we create a new file and save it to the backend. In this case the id changes of the real tab, so the ace tab will no longer find a matching real tab because the id is out of sync.
To fix this we change the crud reducer where we receive the changed tab/file `updateDoEditorSolutionReducer`

There we get the changed tab back from the backend with the new ids, we call

```ts
const storedTabs = AceEditorsStorageHelper.getEditor(doEditorEditorPanelViewEditorId)
```

to get the currently stored ace tabs and

```ts
updateFileReferencesAceStorage(storedTabs, action.payload.solutionVersion.solutionFiles, action.payload.fileIdAssociationMap)
```

which will update the `storedTabs` ids to match the just received ones.

---

As long as the real tabs and ace tabs are in sync the `selectedTabId` of the multi-tab editor *should* work automatically (e.g. when set programmatically).