# install YAPEX

we will install all yapex parts/projects into `/opt/yapex/*`

tested on debian 9 / ubuntu 18

## some useful hints

for unix you can create a ssh tunnel to a remote server via

```
ssh [user]@[server] -L [local port]:[remote server]:[remote port] -N
# or with some verbose information you can add -v (1 level) -v -v (2 levels) or -v -v -v (3 levels)
ssh [user]@[server] -L [local port]:[remote server]:[remote port] -N -v

#e.g.
ssh [user]@[xyz] -L 8080:localhost:8002 -N -v # then any traffic to localhost:8080 is forwarded to the xyz server localhost:8002

#or
ssh [user]@[xyz] -L 8080:[xyz]:8002 -N -v
```


## install dotnet core

goto https://github.com/dotnet/core/blob/master/release-notes/1.1/1.1.11/1.1.11-download.md and download sdk binaries (or runtime, if you want to build the client server locally and only copy fields)

then

```bash
sudo mkdir -p /opt/dotnet
sudo tar zxf [dotnet].tar.gz -C /opt/dotnet
#check if working
cd /opt/dotnet

# create symbolic link from /usr/local/bin/dotnet -> /opt/dotnet/dotnet
sudo ln -s /opt/dotnet/dotnet /usr/local/bin/dotnet

dotnet --version
# should output 1.1.12
```

then also install everything at https://github.com/dotnet/core/blob/master/Documentation/linux-prereqs.md

then
```
# required by nuget to install project dependencies via dotnet restore...
sudo apt-get install libcurl3
```

we use the sdk here (if sdk complains about some `libunwind` is missing install then install `sudo apt-get install libunwind-dev`)

if that fails with `Failed to initialize CoreCLR, HRESULT: 0x80131500` you **maybe** need to install some more libs...
run

```
find /opt/dotnet -name '*.so' -type f -print | xargs ldd | grep 'not found'
```

was missing in ubuntu 18 (but worked anyway)
```
  liblldb-3.9.so.1 => not found
	liblttng-ust.so.0 => not found
```

## install postgres (tested with version 10.x.x)

see https://www.postgresql.org/download/linux/debian/

or just try `sudo apt-get install postgresql`

we will create a new postgres user to interact with the db

```bash
# to enter psql console: sudo -u postgres psql postgres
sudo -u postgres createuser yapex --pwprompt
# note the entered password...

# create the db for yapex
sudo -u postgres createdb yapexDb

# now grant rights to the user
sudo -u postgres psql postgres

# prompt should be psql=# here
grant all privileges on database "yapexDb" to yapex ;

#then exit
\q

# in production you might give only insert, update, delete rights...
# but then you need another user to do database migrations!
```

if you specified another db name or user name you need to adjust the settings in the `appsettings.json` in the client server...
If you only see a `appsettings_example.json` copy it and rename the copy to `appsettings.json`

make sure we have local access

```
sudo vi /etc/postgresql/[version]/main/pg_hba.conf
```

scroll down to something with `socket connection` (table at the very bottom)

and change

```
local   all             all                                     peer
```
to 

```
local   all             all                                     md5
```

## install client server

you can download the client server code locally and only deploy the needed files to run the client server
OR directly clone the repo on the server

we will clone the repo onto the server for this documentation

```bash
sudo apt-get install git
sudo mkdir /opt/yapex/
cd /opt/yapex
# we clone here only the current branch
sudo git clone https://gitlab.informatik.uni-halle.de/Syndrom/ClientServer.git --branch develop

# i use chown to not need to sudo everything inside ClientServer e.g. install dependencies
sudo chown -R [nutzer] ClientServer/
```

**make sure in `src/ClientServer/ClientServer.csproj`**
- the `RuntimeFrameworkVersion` is set to your installed version (1.1.11), you can see your installed version when going into `cd /opt/dotnet/shared/Microsoft.NETCore.App` and look for a folder with `X.X.X`
- `RuntimeIdentifiers` is set to `linux-x64` (for a list see https://docs.microsoft.com/de-de/dotnet/core/rid-catalog)

>1.1.12 is the dotnet sdk/cli version, 1.1.11 is the runtime version

```bash
# install dependencies
cd /opt/yapex/ClientServer
dotnet restore
```

if you changed something in ClientServer.csproj go back to `/opt/yapex/ClientServer` and run `dotnet restore` again!

---

**make sure** you copy the `appsettings_example.json` into `appsettings.json`

**before** you build the server (dotnet publish) you should configure the client server... see section *configuring the client server* below
**for the first run** make sure in `appsettings.json` *IsDbBootstrapEnabled* is set to `true` else the db tables will no be created!

---

make sure in `/opt/yapex/ClientServer/src/ClientServer` only you (and the group) has access to the `appsettings.json` because there the db password is stored

```bash
cd /opt/yapex/ClientServer/src/ClientServer
chmod o-r appsettings.json

# dotnet publish copies this file with it's file permission
```

after that you can build the client server
you can get an idea about the build process from the file `/opt/yapex/ClientServer/src/ClientServer/build/build.sh` it should contain something like (run this in a terminal)

```bash
cd /opt/yapex/ClientServer/src/ClientServer

dotnet publish --configuration Release --framework netcoreapp1.0  -o output/ClientServer

# copy needed script
cp dashboard.php output/ClientServer/dashboard.php
```

the output is found in `output/ClientServer`

in `output/ClientServer` there should be an dir `wwwroot`. This is where we will past the frontend files

then try to run the server by

```bash
cd /opt/yapex/ClientServer/src/ClientServer/output/ClientServer
dotnet ClientServer.dll
```

if that fails with `Failed to initialize CoreCLR, HRESULT: 0x80131500` you **maybe** need to install some more libs...
run

```bash
find /opt/dotnet -name '*.so' -type f -print | xargs ldd | grep 'not found'
```

to find out which and install them

```
liblttng-ust.so.0 => not found
liblldb-3.9.so.1 => not found
```

were missing but it works anyway on ubuntu

### configuring the client server

in `/opt/yapex/ClientServer/src/ClientServer/output/ClientServer` you should fiind a `appsettings.json` file

this contains the config for the client server and is read after every app start

you need to change the following keys: `DbConnectionString`
```json
{
  "IsDebugMode": true,
  "IsDbBootstrapEnabled": true,
  "DbConnectionString": "Username=[USERNAME];Password=[PASSWORD];Host=localhost;Database=[DBNAME];Pooling=true",
  "SystemSalt": "NeverChangeThisAfterFirstStartup",
  "ExternalLoginUrl": "https://uebungsportal.informatik.uni-halle.de/ostepu/DB/DBTransaction/transaction/authentication/redirect/transaction/",
  "MainEntrySite": "/",
  "LoginExternalFirstTimeSite": "/firstLoginExternalLogin.html",
  "RunAssessmentsAsync":  false,
  "DashboardScriptPathRelativeToCurrentDir": "dashboard.php",
  "IsSlave": false
}
```

`IsDbBootstrapEnabled` true to enable create db tables and apply any pending migrations/changes to the database
`MainEntrySite` needs to be the url where the site is running (e.g. via nginx)
`LoginExternalFirstTimeSite` is the url to redirect to after the user needs to set the initial password (only when the user signs in via `ExternalLoginUrl`)
`ExternalLoginUrl` is the login via ostepu

also make sure only eligible users have access to the `appsettings.json` file in
`/opt/yapex/ClientServer/src/ClientServer/appsettings.json` and `/opt/yapex/ClientServer/src/ClientServer/output/ClientServer/appsettings.json`

e.g. run

```bash
chmod o-r /opt/yapex/ClientServer/src/ClientServer/appsettings.json
chmod o-r /opt/yapex/ClientServer/src/ClientServer/output/ClientServer/appsettings.json
```

### creating start script for systemd (optional)

```bash
cd /etc/systemd/system
sudo touch runYapex.service
```

past the following into the file `runYapex.service`

```text
[Unit]
Description=Yapex Backend

[Service]
WorkingDirectory=/opt/yapex/ClientServer/src/ClientServer/output/ClientServer
ExecStart=/opt/dotnet/dotnet /opt/yapex/ClientServer/src/ClientServer/output/ClientServer/ClientServer.dll
Restart=always
RestartSec=10
SyslogIdentifier=dotnet-yapex

[Install]
WantedBy=multi-user.target
```

make sure the file can be executed `sudo chmod +x runYapex.service`

then run to reload the daemon because we created/changed a uni

```bash
sudo systemctl daemon-reload
```

then you can `systemctl status runYapex.service` to get the status
`systemctl start runYapex.service` to start
`systemctl stop runYapex.service` to stop
`systemctl enable runYapex.service` to enable auto start when the server boots
`systemctl disable runYapex.service` to disable auto start when the server boots


when you first start the server you might see `system settings not found, cannot run submissions scheduler.NextRun` in the logs
this is because the scheduler (which runs every X mins) tries to access the system settings (first row in table in the db) but we have no data yet

**on the first start** there are no data in the database.
To automatically create the db schema and basic data you need to make sure in the `appsettings.json` the `IsInitControllerEnabled` is set to true
to fill in the default data open the browser at http://localhost:5000/api/init (some browsers display an json error, this can be ignored) OR run  
 `curl -i -X GET http://localhost:5000/api/init`
*this will only fill in the default data (first user, default group, default role, ...) when there are no (yapex) users in the database!*
After the db was created an you can successfully login you can set the `IsInitControllerEnabled` in `appsettings.json` to false again

## build frontend

for the frontend it is the same... you can build it locally and only cope the files or on the server

for building you will need node to be installed


```bash
cd /opt/yapex
sudo apt update
sudo apt install curl
# debian default nodejs is 4.X ... we want need least 8.x (probably)
sudo curl -sL https://deb.nodesource.com/setup_8.x -o nodesource_setup.sh
sudo bash nodesource_setup.sh

sudo apt-get install -y nodejs
# to make native addons
sudo apt-get install gcc g++ make

# we want nodejs to be called node (to not change the scripts)
#or look where nodejs is installed via 'which nodejs'
sudo ln -s /usr/bin/nodejs /usr/local/bin/node
```

also install yarn (package manager) (npm cannot install some dependencies...)

```bash
curl -sL https://dl.yarnpkg.com/debian/pubkey.gpg | sudo apt-key add -
echo "deb https://dl.yarnpkg.com/debian/ stable main" | sudo tee /etc/apt/sources.list.d/yarn.list
sudo apt-get update && sudo apt-get install yarn
```


```bash
cd /opt/yapex
# we clone here only the current branch
sudo git clone https://gitlab.informatik.uni-halle.de/Syndrom/frontend-react.git --branch develop

# i use chown to not need to sudo everything inside ClientServer e.g. install dependencies
sudo chown -R [user] frontend-react/

#then install dependencies
cd /opt/yapex/frontend-react
yarn install

yarn run build
```

the output is then found in `dist/`

you need to copy the output into the `wwwroot` dir from the client server

```bash
# you might want to clear old stuff first
# cd /opt/yapex/ClientServer/src/ClientServer/output/ClientServer && rm -R wwwroot/*

cp -R /opt/yapex/frontend-react/dist/* /opt/yapex/ClientServer/src/ClientServer/output/ClientServer/wwwroot
```

after this you can start (if it is not running) the client server and open the browser at http://localhost:5000/index_deploy.html (assuming you didn't change the port)

the default admin login is
>id: j
>pw: j

### re-deploy frontend (new version)

When you have a new version of the frontend you need to do

```bash
cp -R /opt/yapex/frontend-react/dist/* /opt/yapex/ClientServer/src/ClientServer/output/ClientServer/wwwroot

#to speed things up (or do it manually) you can copy only the changed files...
# these are normally

# /opt/yapex/frontend-react/dist/index_deploy.html #because the cache buster hash has changed

# when the ts/js code was changed
#/opt/yapex/frontend-react/dist/static/app_bundle.js
#/opt/yapex/frontend-react/dist/static/chunks/vendors~app.chunk.js #needs to be copied because this directly depends on app entry point... (why??)

#when the css was changed
#/opt/yapex/frontend-react/dist/static/styles.css
```

## test server

start the client server and open http://localhost:5000/index_deploy.html and login

and goto menu > `system settings` and change `Test server (urls)` settings to match the port you want to use.
later we will setup nginx and use it as a (reverse proxy) webserver and run our site on port 8002 and the test server on port 8003 so we would need to set
- `Test server url` to `http://localhost:8003/api.php`
- `Submit test server url` to `http://localhost:8003/api.php`
- `Test server stats url` to `http://localhost:8003/stats/`
- `Test server config ui url` to `http://localhost:8003` (should point to phpmyadmin  to manage the test server settings)

### install mysql

(https://dev.mysql.com/doc/mysql-apt-repo-quick-guide/en/ OR https://dbahire.com/how-to-install-mysql-server-on-debian-stretch/)

```
sudo apt-get install mysql-server

#then setup
sudo mysql_secure_installation
```

if some post processing fails you may need to first clear all running mysql processes with

```
sudo kill $(pgrep mysql)
```

if were not ask to set the root pw run

```
sudo mysql_secure_installation
```

if you messed up you can uninstall mysql with
```
#sudo apt-get clean
sudo apt-get remove --purge mysql-server mysql-client mysql-common -y
sudo apt-get autoremove -y
sudo apt-get autoclean
sudo rm -rf /etc/mysql
```

you can now always do `sudo mysql -u root` to login into mysql/mariaDb

login as normal user (not root)... (https://kofler.info/root-login-problem-mit-mariadb/)

>MariaDB führt jetzt nämlich zwei Authentifizierungsverfahren parallel aus: Der DB-Server überprüft, ob Sie als Linux-Benutzer root-Rechte haben UND ob Sie das korrekte Passwort angeben. Das Passwort alleine reicht nicht mehr.

```bash
sudo mysql --user=root mysql

#now you should see the mysql> prompt
select user,host,password,plugin from mysql.user;

update mysql.user set plugin='' where user='root';
flush privileges;
```

### install php

( 5.X or 7.X should be ok) (see https://www.digitalocean.com/community/tutorials/how-to-install-and-secure-phpmyadmin-on-debian-9)

```bash
sudo apt-get install php7.0
sudo apt-get install phpmyadmin php-mbstring php-gettext
sudo phpenmod mbstring
```


>Warning: When the prompt appears, “apache2” is highlighted, but not selected. If you do not hit SPACE to select Apache, the installer will not move the necessary files during installation. Hit SPACE, TAB, and then ENTER to select Apache.

then there should be a  config for phpmyadmin in `/etc/apache2/conf-enabled/phpmyadmin.conf`

if not you probably no selected `apache2` in the configuration...

to rerun the configuration you can uninstall and reinstall phpmyadmin

```bash
sudo apt-get remove --purge phpmyadmin
sudo apt-get install phpmyadmin
```

you probably want to create a new user to manage the mysql database

```bash
sudo mysql --user=root mysql

#now you should see the mysql> prompt
CREATE USER '[DB user]'@'localhost' IDENTIFIED BY '[PASSWORD]';
GRANT ALL PRIVILEGES ON *.* TO '[DB user]'@'localhost' WITH GRANT OPTION;
FLUSH PRIVILEGES;
\q
```

### install test server

clone the test server

```bash
cd /opt/yapex
sudo git clone https://gitlab.informatik.uni-halle.de/Syndrom/TestServer.git
sudo chown -R [user] TestServer
```

then we need to edit the settings in `/opt/yapex/TestServer/config.json`
e.g.

```json
{

    "dbServer": "localhost",
    "dbName": "yapexTestDb",
    "dbUser": "yapex",
    "dbPw": "[PW]",
    "dbPLangTableName": "plangs",
    "dbTransactionTableName": "transactions",
    "useConfigFromDb": true,
    "dbConfigServer": "localhost",
    "dbConfigName": "yapexTestDb",
    "dbConfigUser": "[DB user]",
    "dbConfigPw": "[PW]",
    "dbTableNameConfig": "config",

    "environmentVars": "USER=yapextester\nHOME=/home/yapextester\nLANG=de_DE.UTF-8\nPATH=/usr/local/bin:/usr/bin:/bin:/"
}
```

the user `yapextester` is created via

```bash
sudo adduser yapextester
#pw: tester
```

you can read the `readme.md` for more information about the settings

now make sure only the correct users can access `/opt/yapex/TestServer/config.json`

```bash
sudo chmod o-r config.json

sudo chown yapextester config.json

#then try http://localhost:8003/config.json in thr browser should given 403 forbidden (because it has .json as file extension and we only forward .php)
```

then open phpmyadmin (http://localhost/phpmyadmin/) and create a new db e.g. `yapexTestDb`

you can create the tables as described in the `readme.md` in the test server or import the table from `TestServer/backup/*_server.sql`

the above config specifies that we use the table `config` instead of the local `config.json` file.
You probably need to change the value for `environmentVars` to match your system/needs.

you will need to edit the table `config` and change `workingDirFullPath` to point the the dir used to run tests (the user files will be created there...)

run

```bash
sudo mkdir /opt/yapex/_tests
```

and set `workingDirFullPath` to the created dir (`/opt/yapex/_tests`)

also `runner` needs to be changed, see section **test runner** for more information

then you only need to start a *server* in the directory `/opt/yapex/TestServer/` (we will use nginx with the php-fpm plugin (fastCGI process manager), see nginx section)

## test runner

```bash
cd /opt/yapex/
sudo git clone https://gitlab.informatik.uni-halle.de/Syndrom/LinuxSafeTestRunner.git
sudo chown -R [user] LinuxSafeTestRunner
```

you can also use the `DefaultTestRunnerFullThreaded` (https://gitlab.informatik.uni-halle.de/Syndrom/DefaultTestRunnerFullThreaded.git) but this is only for local testing because it does not run sandboxing commands

when you use the `LinuxSafeTestRunner` you need to change a hardcoded variable `FirejailProfilePath` to point to your firejail config!


### install java sdk

```bash
sudo apt install default-jdk
```

then compile the test runner

```bash
cd /opt/yapex/LinuxSafeTestRunner/src
javac -encoding "UTF-8"  Main.java
```

output will be `.class` files

then we can set this in the configuration of the `TestServer`
open the table `config` in the `yapexTestDb` and edit the column `runner`
the value should be 

```bash
"java" -Dfile.encoding=UTF8 -cp "/opt/yapex/LinuxSafeTestRunner/src" Main
```

you can try executing this command in the terminal which should give you some output like 

>unknown command, length of arguments: 0

then you know the command is correct

## ngin x


```bash
sudo apt-get install nginx
```

>this might fail to start because apache is already using port 80

then comment out all lines in `/etc/nginx/sites-available/default`

create a new entry for our site/app in `/etc/nginx/sites-enabled` (or in /etc/nginx/sites-available/ and then create a symlink)

```bash
sudo touch /etc/nginx/sites-available/yapex
ls /var/run/php/
# there should be a file phpX.Y-fpm.sock
```

and configure the site (at `/etc/nginx/sites-available/yapex`) e.g.

```nginx
server {
  listen 8002;

  #server_name oopsyn.informatik.uni-halle.de;

  #error_page 497 https://oopsyn.informatik.uni-halle.de:8002;

  location /api/ {
      proxy_pass http://localhost:5000;
  }

  location / {
      #return 301 https://yapex.informatik.uni-halle.de;
      root /opt/yapex/ClientServer/src/ClientServer/output/ClientServer/wwwroot;
      index index_deploy.html;
  }

}

server {

  listen 8003;
  root /opt/yapex/TestServer;

   location ~* \.php {
          fastcgi_split_path_info ^(.+\.php)(/.+)$;
          root /opt/yapex/TestServer;
          fastcgi_pass unix:/var/run/php/php[X].[Y]-fpm-yapex.sock;
          include fastcgi.conf;
  }

}
```

then make the site available

```bash
sudo ln -s /etc/nginx/sites-available/yapex /etc/nginx/sites-enabled/
```

then restart nginx

```bash
sudo systemctl restart nginx.service
sudo systemctl status nginx.service
```

then you can open http://localhost:8002 and should see the yapex site (you can use e.g. a ssh tunnel for that)

To get the real user ip (e.g. for exams) we need to add the X-Forwarded-For header via nginx (because else we ould get 127.0.0.1 because nginx is our reverse proxy).
Open `/etc/nginx/nginx.conf` and add to the `http` section
```
...
proxy_set_header X-Forwarded-For $remote_addr;
...
```
this will replace or set the X-Forwarded-For with the real remote addr... (the user might spoof the address badly, see https://serverfault.com/questions/314574/nginx-real-ip-header-and-x-forwarded-for-seems-wrong [ngx_http_realip_module] to prevent this)


you can also check the test-server is running properly when you open `http://localhost:8003/[a test-server.php file]` in the browser e.g. `http://localhost:8003/index.php`

### we switch from apach2 to nginx as webserver for phpMyAdmin

```bash
cd /etc/nginx/sites-available
ls /var/run/php/
# there should be a file phpX.Y-fpm.sock
sudo touch phpMyAdmin
```

open `/etc/nginx/sites-available/phpMyAdmin`
and insert

```nginx
server {

  listen 8000;
  root /usr/share/phpmyadmin;
  index index.php;

   location ~* \.php {
          fastcgi_split_path_info ^(.+\.php)(/.+)$;
          fastcgi_pass unix:/var/run/php/php[X].[Y]-fpm.sock;
          include fastcgi.conf;
  }

}
```

then make the site available

```bash
sudo ln -s /etc/nginx/sites-available/phpMyAdmin /etc/nginx/sites-enabled/
```

then stop apache2 and restart nginx

```bash
sudo systemctl stop apache2
sudo systemctl disable apache2 # if we don't need apacahe anymore
sudo systemctl restart nginx
```

then you can open the browser at http://localhost:8000 and should see the phpmyadmin site

### configure php-fpm

we will use apache php-fpm plugin to control the php processes

```bash
sudo apt install php-fpm
```

we already configured nginx to process php requests in the file `/etc/nginx/sites-available` to everything should work now

restart nginx via

```bash
sudo systemctl restart nginx.service
```

now almost everything should run. when you create an exercise in the ui and try to run a test you will get the following error: `could not create directory`.

This is because the executing user (for the fpm) has no permission in the working dir `/opt/yapex/_tests`
so we need to change the executing user...

for security reasons we created the yapextester user

then change the fpm config

```bash
cd /etc/php/[version]/fpm/pool.d
sudo cp www.conf www.conf.backup # just to be safe
sudo cp www.conf yapex_runner.conf
sudo vi www.conf

#change the lines
#; pool name ('www' here) toy

[yapex]

...

# user = www-data
user = yapextester
group = yapextester

...
#listen = /run/php/php7.2-fpm.sock
listen = /run/php/php[X].[Y]-fpm-yapex.sock
```

then restart php-fcm

```bash
systemctl restart php[X].[Y]-fpm.service
```

*we could let the default www-data user but we got for a custom user here*

then we need to give this user permission to write to `/opt/yapex/_tests/`

```bash
cd /opt/yapex
sudo chown -R yapextester _tests
```

if you used the `LinuxSafeTestRunner` you still need to setup firejal (sandbox program)
in order to run tests

remember that the new user `yapextester` needs write access to `TestServer` and `LinuxSafeTestRunner`

*[TODO-dev maybe we should run the runner with a different user that does not have permission to read the config from the `TestServer`??]*


### firejail (required for LinuxSafeTestRunner)

the `LinuxSafeTestRunner` includes a very basic firejail config in `firejail/run.profile`

```bash
sudo apt-get install firejail
# check with firejail --version
```

TODO lxc privilege issue??

```
Error mount read-only: fs.c:574 fs_rdonly: Permission denied
Error: cannot establish communication with the parent, exiting...
```



https://github.com/netblue30/firejail/issues/1384






## Adding a new programming language

add the new plang in the app ui (menu > system settings)

add the new plang in the test server settings (open phpmyadmin, the `plangs` table)
insert the right commands here (see `TestServer/readme.md` for more help)

then enable the compiler in the firejail config in `LinuxSafeTestRunner/firejail/run.profile`

TODO which line??
